


#include "Handler.h"



// ======================================================================================================

// This class realizes a simple unidirectional list of handlers

class TListOfHandlers
{
	private:

		THandler * fHead;		// ptr to the first element in the list, or 0

	public:

		THandler * GetHead( void ) { return fHead; }

	public:

		///////////////////////////////////////////////////////////
		// Class constructor
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			none
		//		
		// REMARKS:
		//			Creates an empty list
		//		
		TListOfHandlers( void ) : fHead( 0 ), fIteratorCurrentPosition( 0 ) {}

		///////////////////////////////////////////////////////////
		// Class destructor
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			none
		//		
		// REMARKS:
		//			All possesed objects are now destroyed
		//		
		virtual ~TListOfHandlers( void )
		{
			THandler * next = 0;
			for( THandler * p = fHead; p != 0; p = next )
			{
				next = p->GetNext();
				delete p;
			}
		}

	public:

		///////////////////////////////////////////////////////////
		// This function adds a new handler to the list.
		// From this time on, the new handler is governed
		// by the list.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			orphan_handler - a pointer to the THandler
		//				object (or derived)
		//		
		// OUTPUT:
		//			none
		//		
		// REMARKS:
		//			The orphan_handler object is orphaned on
		//				behalf of this list, so the list
		//				will destroy it in its destructor.
		//		
		void Add2Head( THandler * orphan_handler );


	public:

		// Internal iterators

		///////////////////////////////////////////////////////////
		// This function returns ptr to the first element in 
		// the list and it starts the iteration process.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			ptr to the head
		//		
		// REMARKS:
		//		
		//		
		THandler * GetFirst( void );

		///////////////////////////////////////////////////////////
		// This function returns ptr to the next
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//		
		//		
		// OUTPUT:
		//		
		//		
		// REMARKS:
		//		
		//		
		THandler * GetNext( void );

	private:

		THandler * fIteratorCurrentPosition;		// used during iteration - contains ptr
													// to the currently pointed to object

};

